@@ -43,7 +43,7 @@
 #include <gconf/gconf-client.h>
 
 #define GNOME_DESKTOP_USE_UNSTABLE_API
-#include <libgnome/gnome-desktop-utils.h>
+#include <libgnome-desktop/gnome-desktop-utils.h>
 
 #include "calendar-window.h"
 
@@ -84,7 +84,6 @@ struct _CalendarWindowPrivate {
 	char       *prefs_dir;
 
 	gboolean     invert_order;
-	gboolean     utc_time;
 	gboolean     show_weeks;
 	time_t      *current_time;
 
@@ -119,7 +118,6 @@ G_DEFINE_TYPE (CalendarWindow, calendar_window, GTK_TYPE_WINDOW)
 enum {
 	PROP_0,
 	PROP_INVERTORDER,
-	PROP_UTCTIME,
 	PROP_SHOWWEEKS,
 #ifdef HAVE_LIBECAL
 	PROP_TIMEFORMAT,
@@ -142,6 +140,15 @@ static GtkWidget * create_hig_frame 		  (CalendarWindow *calwin,
 
 #ifdef HAVE_LIBECAL
 
+/*
+ * Set the DISPLAY variable, to be use by g_spawn_async.
+ */
+static void
+set_environment (gpointer display)
+{
+	g_setenv ("DISPLAY", display, TRUE);
+}
+
 static void
 clock_launch_calendar_tasks_app (CalendarWindow *calwin,
 				 const char     *key_program,
@@ -156,6 +163,7 @@ clock_launch_calendar_tasks_app (CalendarWindow *calwin,
 	GdkScreen  *screen;
 	GError     *error;
 	gboolean    result;
+	char       *display;
 
 	key = g_strdup_printf ("%s%s", key_program, "/exec");
 	program = gconf_client_get_string (calwin->priv->gconfclient,
@@ -196,15 +204,18 @@ clock_launch_calendar_tasks_app (CalendarWindow *calwin,
 	if (terminal)
 		gnome_desktop_prepend_terminal_to_vector (&argc, &argv);
 
-	result = gdk_spawn_on_screen (screen,
-				      NULL, /* working directory */
-				      argv,
-				      NULL, /* envp */
-				      G_SPAWN_SEARCH_PATH,
-				      NULL, /* child setup func */
-				      NULL, /* user data */
-				      NULL, /* child pid */
-				      &error);
+	display = gdk_screen_make_display_name (screen);
+
+	result = g_spawn_async (NULL, /* working directory */
+			        argv,
+				NULL, /* envp */
+				G_SPAWN_SEARCH_PATH,
+				set_environment, /* child setup func */
+				&display, /* user data for child setup */
+				NULL,     /* child pid */
+				&error);
+
+	g_free (display);
 
 	if (!result) {
 		g_printerr ("Cannot launch calendar/tasks application: %s\n",
@@ -281,8 +292,7 @@ format_time (ClockFormat format,
              time_t      t,
              guint       year,
              guint       month,
-             guint       day,
-             gboolean    use_utc)
+             guint       day)
 {
         struct tm *tm;
         char      *time_format;
@@ -291,10 +301,7 @@ format_time (ClockFormat format,
         if (!t)
                 return NULL;
 
-        if (use_utc)
-                tm = gmtime (&t);
-        else
-                tm = localtime (&t);
+	tm = localtime (&t);
         if (!tm)
                 return NULL;
 
@@ -456,8 +463,10 @@ is_appointment (GtkTreeModel *model,
 	gtk_tree_model_get (model, iter, APPOINTMENT_COLUMN_URI, &uri, -1);
 	if (uri)
 		return (g_ascii_strcasecmp (uri, "file")   == 0 ||
+			g_ascii_strcasecmp (uri, "local")   == 0 ||
 			g_ascii_strcasecmp (uri, "webcal") == 0 ||
 			g_ascii_strcasecmp (uri, "caldav") == 0 ||
+			g_ascii_strcasecmp (uri, "mapi")   == 0 ||
 			g_ascii_strcasecmp (uri, "exchange")  == 0 ||
 			g_ascii_strcasecmp (uri, "groupwise") == 0 ||
 			g_ascii_strcasecmp (uri, "google") == 0);
@@ -975,8 +984,7 @@ handle_appointments_changed (CalendarWindow *calwin)
                 else
                         start_text = format_time (calwin->priv->time_format,
                                                   appointment->start_time,
-                                                  year, month, day,
-                                                  calwin->priv->utc_time);
+                                                  year, month, day);
 
 
                 gtk_list_store_append (calwin->priv->appointments_model,
@@ -1305,48 +1313,34 @@ calendar_month_selected (GtkCalendar    *calendar,
         handle_tasks_changed (calwin);
 }
 
-typedef struct
-{
-        GtkWidget *calendar;
-        GtkWidget *tree;
-} ConstraintData;
-
-static void
-constrain_list_size (GtkWidget      *widget,
-                     GtkRequisition *requisition,
-                     ConstraintData *constraint)
-{
-        GtkRequisition req;
-        int            screen_h;
-        int            max_height;
-
-        /* constrain width to the calendar width */
-        gtk_widget_size_request (constraint->calendar, &req);
-        requisition->width = MIN (requisition->width, req.width);
-
-	screen_h = gdk_screen_get_height (gtk_widget_get_screen (widget));
-        /* constrain height to be the tree height up to a max */
-        max_height = (screen_h - req.height) / 3;
-        gtk_widget_size_request (constraint->tree, &req);
-
-        requisition->height = MIN (req.height, max_height);
-        requisition->height += 2 * widget->style->ythickness;
-}
-
 static void
 setup_list_size_constraint (GtkWidget *widget,
                             GtkWidget *calendar,
                             GtkWidget *tree)
 {
-        ConstraintData *constraint;
+	GtkRequisition   req;
+	GtkStyleContext *context;
+	GtkStateFlags    state;
+	GtkBorder        padding;
+	int              screen_h;
+	int              max_height;
+	int              req_height;
 
-        constraint           = g_new0 (ConstraintData, 1);
-        constraint->calendar = calendar;
-        constraint->tree     = tree;
+	/* constrain width to the calendar width */
+	gtk_widget_get_preferred_size (calendar, &req, NULL);
 
-        g_signal_connect_data (widget, "size-request",
-                               G_CALLBACK (constrain_list_size), constraint,
-                               (GClosureNotify) g_free, 0);
+	screen_h = gdk_screen_get_height (gtk_widget_get_screen (widget));
+        /* constrain height to be the tree height up to a max */
+	max_height = (screen_h - req.height) / 3;
+	gtk_widget_get_preferred_size (tree, &req, NULL);
+
+	state = gtk_widget_get_state_flags (widget);
+	context = gtk_widget_get_style_context (widget);
+	gtk_style_context_get_padding (context, state, &padding);
+
+	req_height = MIN (req.height, max_height);
+	req_height += padding.top + padding.bottom;
+	gtk_widget_set_size_request (widget, req.width, req_height);
 }
 
 static void
@@ -1544,10 +1538,7 @@ calendar_window_create_calendar (CalendarWindow *calwin)
 		options &= ~(GTK_CALENDAR_SHOW_WEEK_NUMBERS);
 	gtk_calendar_set_display_options (GTK_CALENDAR (calendar), options);
 
-	if (calwin->priv->utc_time)
-		tm = gmtime (calwin->priv->current_time);
-	else
-		tm = localtime (calwin->priv->current_time);
+	tm = localtime (calwin->priv->current_time);
 
         gtk_calendar_select_month (GTK_CALENDAR (calendar),
                                    tm->tm_mon,
@@ -1752,10 +1743,6 @@ calendar_window_get_property (GObject    *object,
 		g_value_set_boolean (value,
 				     calendar_window_get_invert_order (calwin));
 		break;
-	case PROP_UTCTIME:
-		g_value_set_boolean (value,
-				     calendar_window_get_utc_time (calwin));
-		break;
 	case PROP_SHOWWEEKS:
 		g_value_set_boolean (value,
 				     calendar_window_get_show_weeks (calwin));
@@ -1797,10 +1784,6 @@ calendar_window_set_property (GObject       *object,
 		calendar_window_set_invert_order (calwin,
 						  g_value_get_boolean (value));
 		break;
-	case PROP_UTCTIME:
-		calendar_window_set_utc_time (calwin,
-					      g_value_get_boolean (value));
-		break;
 	case PROP_SHOWWEEKS:
 		calendar_window_set_show_weeks (calwin,
 						g_value_get_boolean (value));
@@ -1826,7 +1809,7 @@ calendar_window_set_property (GObject       *object,
 }
 
 static void
-calendar_window_destroy (GtkObject *object)
+calendar_window_dispose (GObject *object)
 {
 #ifdef HAVE_LIBECAL
 	CalendarWindow *calwin;
@@ -1866,20 +1849,18 @@ calendar_window_destroy (GtkObject *object)
 	calwin->priv->gconfclient = NULL;
 #endif /* HAVE_LIBECAL */
 
-	GTK_OBJECT_CLASS (calendar_window_parent_class)->destroy (object);
+	G_OBJECT_CLASS (calendar_window_parent_class)->dispose (object);
 }
 
 static void
 calendar_window_class_init (CalendarWindowClass *klass)
 {
 	GObjectClass   *gobject_class   = G_OBJECT_CLASS (klass);
-	GtkObjectClass *gtkobject_class = GTK_OBJECT_CLASS (klass);
 
 	gobject_class->constructor = calendar_window_constructor;
 	gobject_class->get_property = calendar_window_get_property;
         gobject_class->set_property = calendar_window_set_property;
-
-	gtkobject_class->destroy = calendar_window_destroy;
+	gobject_class->dispose = calendar_window_dispose;
 
 	g_type_class_add_private (klass, sizeof (CalendarWindowPrivate));
 
@@ -1901,15 +1882,6 @@ calendar_window_class_init (CalendarWindowClass *klass)
 				      FALSE,
 				      G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
 
-	g_object_class_install_property (
-		gobject_class,
-		PROP_UTCTIME,
-		g_param_spec_boolean ("utc-time",
-				      "UTC Time",
-				      "Use UTC time as a basis for the calendar",
-				      FALSE,
-				      G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
-
 	g_object_class_install_property (
 		gobject_class,
 		PROP_SHOWWEEKS,
@@ -1978,15 +1950,12 @@ calendar_window_new (time_t     *static_current_time,
 	CalendarWindow *calwin;
 
 	calwin = g_object_new (CALENDAR_TYPE_WINDOW,
+			       "type", GTK_WINDOW_TOPLEVEL,
 			       "current-time", static_current_time,
 			       "invert-order", invert_order,
 			       "prefs-dir", prefs_dir,
 			       NULL);
 
-	/* It doesn't work if we do this in calendar_window_init(), so do it
-	 * here */
-	GTK_WINDOW (calwin)->type = GTK_WINDOW_TOPLEVEL;
-
 	return GTK_WIDGET (calwin);
 }
 
@@ -2030,33 +1999,6 @@ calendar_window_set_invert_order (CalendarWindow *calwin,
 	g_object_notify (G_OBJECT (calwin), "invert-order");
 }
 
-gboolean
-calendar_window_get_utc_time (CalendarWindow *calwin)
-{
-	g_return_val_if_fail (CALENDAR_IS_WINDOW (calwin), FALSE);
-
-	return calwin->priv->utc_time;
-}
-
-void
-calendar_window_set_utc_time (CalendarWindow *calwin,
-			      gboolean        utc_time)
-{
-	g_return_if_fail (CALENDAR_IS_WINDOW (calwin));
-
-	if (utc_time == calwin->priv->utc_time)
-		return;
-
-	calwin->priv->utc_time = utc_time;
-#ifdef HAVE_LIBECAL
-	/* Time to display for appointments has changed */
-	if (calwin->priv->appointments_model)
-		handle_appointments_changed (calwin);
-#endif
-
-	g_object_notify (G_OBJECT (calwin), "utc-time");
-}
-
 gboolean
 calendar_window_get_show_weeks (CalendarWindow *calwin)
 {
