@@ -31,6 +31,7 @@
 #include <time.h>
 
 #include <cairo.h>
+#include <cairo-xlib.h>
 
 #include <glib/gi18n.h>
 #include <glib-object.h>
@@ -73,8 +74,8 @@ typedef struct {
 	GtkWidget         *frame;
 	GtkWidget         *drawing_area;
 	GtkRequisition     requisition;
-	GdkRectangle       prev_allocation;
-	GdkPixmap         *pixmap;
+	GtkAllocation      prev_allocation;
+	cairo_surface_t   *surface;
 	guint              timeout;
 	int                current_frame;
 	gboolean           in_applet;
@@ -113,7 +114,7 @@ typedef struct {
 
 
 static gboolean load_fish_image          (FishApplet *fish);
-static void     update_pixmap            (FishApplet *fish);
+static void     update_surface           (FishApplet *fish);
 static void     something_fishy_going_on (FishApplet *fish,
 					  const char *message);
 static void     display_fortune_dialog   (FishApplet *fish);
@@ -872,6 +873,15 @@ fish_read_output (GIOChannel   *source,
 	return (status != G_IO_STATUS_EOF);
 }
 
+/*
+ * Set the DISPLAY variable, to be use by g_spawn_async.
+ */
+static void
+set_environment (gpointer display)
+{
+	g_setenv ("DISPLAY", display, TRUE);
+}
+
 static void 
 display_fortune_dialog (FishApplet *fish)
 {
@@ -881,6 +891,8 @@ display_fortune_dialog (FishApplet *fish)
 	const char  *charset;
 	int          argc;
 	char       **argv;
+	GdkScreen   *screen;
+	char        *display;
 
 	/* if there is still a pipe, close it */
 	if (fish->source_id)
@@ -908,8 +920,6 @@ display_fortune_dialog (FishApplet *fish)
 
 		gtk_window_set_icon_name (GTK_WINDOW (fish->fortune_dialog),
 					  FISH_ICON);
-		gtk_dialog_set_has_separator (
-			GTK_DIALOG (fish->fortune_dialog), FALSE);
 
 		gtk_dialog_set_default_response (
 			GTK_DIALOG (fish->fortune_dialog), GTK_RESPONSE_CLOSE);
@@ -1000,11 +1010,22 @@ display_fortune_dialog (FishApplet *fish)
 
 	clear_fortune_text (fish);
 
-	gdk_spawn_on_screen_with_pipes (gtk_widget_get_screen (GTK_WIDGET (fish)),
-					NULL, argv, NULL,
-					G_SPAWN_SEARCH_PATH|G_SPAWN_STDERR_TO_DEV_NULL,
-					NULL, NULL, NULL, NULL, &output, NULL,
-					&error);
+	screen = gtk_widget_get_screen (GTK_WIDGET (fish));
+	display = gdk_screen_make_display_name (screen);
+
+	g_spawn_async_with_pipes (NULL, /* working directory */
+				  argv,
+				  NULL, /* envp */
+				  G_SPAWN_SEARCH_PATH|G_SPAWN_STDERR_TO_DEV_NULL,
+				  set_environment,
+				  &display,
+				  NULL, /* child pid */
+				  NULL, /* stdin */
+				  &output,
+				  NULL, /* stderr */
+				  &error);
+
+	g_free (display);
 
 	if (error) {
 		char *message;
@@ -1095,7 +1116,7 @@ image_changed_notify (GConfClient *client,
 	fish->image = g_strdup (value);
 
 	load_fish_image (fish);
-	update_pixmap (fish);
+	update_surface (fish);
 
 	if (fish->image_chooser) {
 		char *path_gconf;
@@ -1158,7 +1179,7 @@ n_frames_changed_notify (GConfClient *client,
 	if (fish->n_frames <= 0)
 		fish->n_frames = 1;
 
-	update_pixmap (fish);
+	update_surface (fish);
 
 	if (fish->frames_spin &&
 	    gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (fish->frames_spin)) != fish->n_frames)
@@ -1254,13 +1275,13 @@ check_april_fools (FishApplet *fish)
 	     tm->tm_mday != fools_day   ||
 	     tm->tm_hour >= fools_hour_end)) {
 		fish->april_fools = FALSE;
-		update_pixmap (fish);
+		update_surface (fish);
 	} else if (tm->tm_mon  == fools_month    &&
 		 tm->tm_mday == fools_day        &&
 		 tm->tm_hour >= fools_hour_start &&
 		 tm->tm_hour <= fools_hour_end) {
 		fish->april_fools = TRUE;
-		update_pixmap (fish);
+		update_surface (fish);
 	}
 }
 
@@ -1337,7 +1358,7 @@ rotate_changed_notify (GConfClient *client,
 
 	if (fish->orientation == PANEL_APPLET_ORIENT_LEFT ||
 	    fish->orientation == PANEL_APPLET_ORIENT_RIGHT)
-		update_pixmap (fish);
+		update_surface (fish);
 
 	if (fish->rotate_toggle &&
 	    gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (fish->rotate_toggle)) != fish->rotate)
@@ -1456,7 +1477,7 @@ load_fish_image (FishApplet *fish)
 }
 
 static void
-update_pixmap (FishApplet *fish)
+update_surface (FishApplet *fish)
 {
 	GtkWidget     *widget = fish->drawing_area;
 	GtkAllocation  allocation;
@@ -1507,21 +1528,27 @@ update_pixmap (FishApplet *fish)
 		}
 	}
 
+	gtk_widget_set_size_request (fish->drawing_area,
+				     fish->requisition.width,
+				     fish->requisition.height);
+
 	g_assert (width != -1 && height != -1);
 
 	if (width == 0 || height == 0)
 		return;
 
-	if (fish->pixmap)
-		g_object_unref (fish->pixmap);
-	fish->pixmap = gdk_pixmap_new (gtk_widget_get_window (widget),
-				       width, height, -1);
+	if (fish->surface)
+		cairo_surface_destroy (fish->surface);
+	fish->surface = gdk_window_create_similar_surface (
+			               gtk_widget_get_window (widget),
+			               CAIRO_CONTENT_COLOR_ALPHA,
+			               width, height);
 
 	gtk_widget_queue_resize (widget);
 
 	g_assert (pixbuf_width != -1 && pixbuf_height != -1);
 
-	cr = gdk_cairo_create (fish->pixmap);
+	cr = cairo_create (fish->surface);
 
 	cairo_set_source_rgb (cr, 1, 1, 1);
 	cairo_paint (cr);
@@ -1569,57 +1596,44 @@ update_pixmap (FishApplet *fish)
 }
 
 static gboolean
-fish_applet_expose_event (GtkWidget      *widget,
-			  GdkEventExpose *event,
-			  FishApplet     *fish)
+fish_applet_draw (GtkWidget  *widget,
+		  cairo_t    *cr,
+		  FishApplet *fish)
 {
-	GdkWindow    *window;
-	GtkStyle     *style;
-	GtkStateType  state;
+	GdkWindow *window;
 	int width, height;
 	int src_x, src_y;
 
-	g_return_val_if_fail (fish->pixmap != NULL, FALSE);
+	g_return_val_if_fail (fish->surface != NULL, FALSE);
 
 	g_assert (fish->n_frames > 0);
 
 	window = gtk_widget_get_window (widget);
-	style = gtk_widget_get_style (widget);
-	state = gtk_widget_get_state (widget);
 
-	gdk_drawable_get_size (fish->pixmap, &width, &height);
+	width = cairo_xlib_surface_get_width (fish->surface);
+	height = cairo_xlib_surface_get_height (fish->surface);
 
-	src_x = event->area.x;
-	src_y = event->area.y;
+	src_x = 0;
+	src_y = 0;
 
 	if (fish->rotate) {
 		if (fish->orientation == PANEL_APPLET_ORIENT_RIGHT)
-			src_y += ((height * (fish->n_frames - 1 - fish->current_frame)) / fish->n_frames);
+			src_y = ((height * (fish->n_frames - 1 - fish->current_frame)) / fish->n_frames);
 		else if (fish->orientation == PANEL_APPLET_ORIENT_LEFT)
-			src_y += ((height * fish->current_frame) / fish->n_frames);
+			src_y = ((height * fish->current_frame) / fish->n_frames);
 		else
-			src_x += ((width * fish->current_frame) / fish->n_frames);
+			src_x = ((width * fish->current_frame) / fish->n_frames);
 	} else
-		src_x += ((width * fish->current_frame) / fish->n_frames);
+		src_x = ((width * fish->current_frame) / fish->n_frames);
 
-	gdk_draw_drawable (window,
-			   style->fg_gc [state],
-			   fish->pixmap,
-			   src_x, src_y,
-			   event->area.x, event->area.y,
-			   event->area.width, event->area.height);
+        cairo_save (cr);
+        cairo_set_source_surface (cr, fish->surface, -src_x, -src_y);
+        cairo_paint (cr);
+        cairo_restore (cr);
 
         return FALSE;
 }
 
-static void
-fish_applet_size_request (GtkWidget      *widget,
-			  GtkRequisition *requisition,
-			  FishApplet     *fish)
-{
-	*requisition = fish->requisition;
-}
-
 static void
 fish_applet_size_allocate (GtkWidget     *widget,
 			   GtkAllocation *allocation,
@@ -1631,7 +1645,7 @@ fish_applet_size_allocate (GtkWidget     *widget,
 
 	if (widget_allocation.width  != fish->prev_allocation.width ||
 	    widget_allocation.height != fish->prev_allocation.height)
-		update_pixmap (fish);
+		update_surface (fish);
 
 	fish->prev_allocation = *allocation;
 }
@@ -1640,17 +1654,17 @@ static void
 fish_applet_realize (GtkWidget  *widget,
 		     FishApplet *fish)
 {
-	if (!fish->pixmap)
-		update_pixmap (fish);
+	if (!fish->surface)
+		update_surface (fish);
 }
 
 static void
 fish_applet_unrealize (GtkWidget  *widget,
 		       FishApplet *fish)
 {
-	if (fish->pixmap)
-		g_object_unref (fish->pixmap);
-	fish->pixmap = NULL;
+	if (fish->surface)
+		cairo_surface_destroy (fish->surface);
+	fish->surface = NULL;
 }
 
 static void
@@ -1664,8 +1678,8 @@ fish_applet_change_orient (PanelApplet       *applet,
 
 	fish->orientation = orientation;
 
-	if (fish->pixmap)
-		update_pixmap (fish);
+	if (fish->surface)
+		update_surface (fish);
 }
 
 static void
@@ -1696,12 +1710,12 @@ handle_keypress (GtkWidget   *widget,
 		 FishApplet  *fish)
 {
 	switch (event->keyval) {
-	case GDK_space:
-	case GDK_KP_Space:
-	case GDK_Return:
-	case GDK_KP_Enter:
-	case GDK_ISO_Enter:
-	case GDK_3270_Enter:
+	case GDK_KEY_space:
+	case GDK_KEY_KP_Space:
+	case GDK_KEY_Return:
+	case GDK_KEY_KP_Enter:
+	case GDK_KEY_ISO_Enter:
+	case GDK_KEY_3270_Enter:
 		if (fish->april_fools) {
 			change_water (fish);
 			return TRUE;
@@ -1795,12 +1809,10 @@ setup_fish_widget (FishApplet *fish)
 			  G_CALLBACK (fish_applet_realize), fish);
 	g_signal_connect (fish->drawing_area, "unrealize",
 			  G_CALLBACK (fish_applet_unrealize), fish);
-	g_signal_connect (fish->drawing_area, "size-request",
-			  G_CALLBACK (fish_applet_size_request), fish);
 	g_signal_connect (fish->drawing_area, "size-allocate",
 			  G_CALLBACK (fish_applet_size_allocate), fish);
-	g_signal_connect (fish->drawing_area, "expose-event",
-			  G_CALLBACK (fish_applet_expose_event), fish);
+	g_signal_connect (fish->drawing_area, "draw",
+			  G_CALLBACK (fish_applet_draw), fish);
 
 	gtk_widget_add_events (widget, GDK_ENTER_NOTIFY_MASK | 
 				       GDK_LEAVE_NOTIFY_MASK |
@@ -1819,7 +1831,7 @@ setup_fish_widget (FishApplet *fish)
 
 	load_fish_image (fish);
 
-	update_pixmap (fish);
+	update_surface (fish);
 
 	setup_timeout (fish);
 
@@ -1970,7 +1982,7 @@ fishy_factory (PanelApplet *applet,
 }
 
 static void
-fish_applet_destroy (GtkObject *object)
+fish_applet_dispose (GObject *object)
 {
 	FishApplet *fish = (FishApplet *) object;
 	int         i;
@@ -2002,9 +2014,9 @@ fish_applet_destroy (GtkObject *object)
 		g_object_unref (fish->client);
 	fish->client = NULL;
 
-	if (fish->pixmap)
-		g_object_unref (fish->pixmap);
-	fish->pixmap = NULL;
+	if (fish->surface)
+		cairo_surface_destroy (fish->surface);
+	fish->surface = NULL;
 
 	if (fish->pixbuf)
 		g_object_unref (fish->pixbuf);
@@ -2028,7 +2040,7 @@ fish_applet_destroy (GtkObject *object)
 
 	fish_close_channel (fish);
 
-	GTK_OBJECT_CLASS (parent_class)->destroy (object);
+	G_OBJECT_CLASS (parent_class)->dispose (object);
 }
 
 static void
@@ -2050,7 +2062,7 @@ fish_applet_instance_init (FishApplet      *fish,
 
 	fish->frame         = NULL;
 	fish->drawing_area  = NULL;
-	fish->pixmap        = NULL;
+	fish->surface       = NULL;
 	fish->timeout       = 0;
 	fish->current_frame = 0;
 	fish->in_applet     = FALSE;
@@ -2101,14 +2113,14 @@ fish_applet_instance_init (FishApplet      *fish,
 static void
 fish_applet_class_init (FishAppletClass *klass)
 {
-	PanelAppletClass *applet_class    = (PanelAppletClass *) klass;
-	GtkObjectClass   *gtkobject_class = (GtkObjectClass *) klass;
+	PanelAppletClass *applet_class  = (PanelAppletClass *) klass;
+	GObjectClass     *gobject_class = (GObjectClass *) klass;
 
 	parent_class = g_type_class_peek_parent (klass);
 
 	applet_class->change_orient = fish_applet_change_orient;
 
-	gtkobject_class->destroy = fish_applet_destroy;
+	gobject_class->dispose = fish_applet_dispose;
 
 	init_fools_day ();
 }
