@@ -177,11 +177,9 @@ static const GtkActionEntry menu_actions [] = {
 };
 
 static void
-applet_change_background (PanelApplet               *applet,
-                          PanelAppletBackgroundType  type,
-                          GdkColor                  *color,
-                          GdkPixmap                 *pixmap,
-                          AppletData                *data)
+applet_change_background (PanelApplet     *applet,
+                          cairo_pattern_t *pattern,
+                          AppletData      *data)
 {
   na_tray_force_redraw (data->tray);
 }
@@ -210,45 +208,24 @@ free_applet_data (AppletData *data)
   g_slice_free (AppletData, data);
 }
 
-static inline void
-force_no_focus_padding (GtkWidget *widget)
+static void
+on_applet_realized (GtkWidget *widget,
+                    gpointer   user_data)
 {
-  static gboolean first_time = TRUE;
-
-  if (first_time)
-    {
-      gtk_rc_parse_string ("\n"
-                           "   style \"na-tray-style\"\n"
-                           "   {\n"
-                           "      GtkWidget::focus-line-width=0\n"
-                           "      GtkWidget::focus-padding=0\n"
-                           "   }\n"
-                           "\n"
-                           "    widget \"*.na-tray\" style \"na-tray-style\"\n"
-                           "\n");
-      first_time = FALSE;
-    }
-
-  gtk_widget_set_name (widget, "na-tray");
-}
-
-static gboolean
-applet_factory (PanelApplet *applet,
-                const gchar *iid,
-                gpointer     user_data)
-{
-  NaTray         *tray;
+  PanelApplet    *applet;
   AppletData     *data;
+  NaTray         *tray;
   GtkActionGroup *action_group;
   gchar          *ui_path;
-  AtkObject      *atko;
 
-  if (!(strcmp (iid, "NotificationArea") == 0 ||
-        strcmp (iid, "SystemTrayApplet") == 0))
-    return FALSE;
+  applet = PANEL_APPLET (widget);
+  data = g_object_get_data (G_OBJECT (widget), "system-tray-data");
+
+  if (data != NULL)
+    return;
 
   tray = na_tray_new_for_screen (gtk_widget_get_screen (GTK_WIDGET (applet)),
-				 get_orientation_from_applet (applet));
+                                 get_orientation_from_applet (applet));
 
   data = g_slice_new (AppletData);
   data->applet = applet;
@@ -260,41 +237,81 @@ applet_factory (PanelApplet *applet,
                           data,
                           (GDestroyNotify) free_applet_data);
 
+  g_signal_connect (applet, "change_orient",
+                    G_CALLBACK (applet_change_orientation), data);
+  g_signal_connect (applet, "change_background",
+                    G_CALLBACK (applet_change_background), data);
+  g_signal_connect (applet, "destroy",
+                    G_CALLBACK (applet_destroy), data);
+
+  gtk_container_add (GTK_CONTAINER (applet), GTK_WIDGET (tray));
+  gtk_widget_show (GTK_WIDGET (tray));
+
+  action_group = gtk_action_group_new ("ClockApplet Menu Actions");
+  gtk_action_group_set_translation_domain (action_group, GETTEXT_PACKAGE);
+  gtk_action_group_add_actions (action_group,
+                                menu_actions,
+                                G_N_ELEMENTS (menu_actions),
+                                data);
+  ui_path = g_build_filename (NOTIFICATION_AREA_MENU_UI_DIR, "notification-area-menu.xml", NULL);
+  panel_applet_setup_menu_from_file (applet,
+                                     ui_path, action_group);
+  g_free (ui_path);
+  g_object_unref (action_group);
+
+}
+
+static inline void
+force_no_focus_padding (GtkWidget *widget)
+{
+  GtkCssProvider *provider;
+
+  provider = gtk_css_provider_new ();
+  gtk_css_provider_load_from_data (provider,
+                                   "#na-tray {\n"
+                                   " -GtkWidget-focus-line-width: 0px;\n"
+                                   " -GtkWidget-focus-padding: 0px;\n"
+				   "}",
+                                   -1, NULL);
+  gtk_style_context_add_provider (gtk_widget_get_style_context (widget),
+                                  GTK_STYLE_PROVIDER (provider),
+                                  GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
+  g_object_unref (provider);
+
+  gtk_widget_set_name (widget, "na-tray");
+}
+
+static gboolean
+applet_factory (PanelApplet *applet,
+                const gchar *iid,
+                gpointer     user_data)
+{
+  AtkObject *atko;
+
+  if (!(strcmp (iid, "NotificationArea") == 0 ||
+        strcmp (iid, "SystemTrayApplet") == 0))
+    return FALSE;
+
+  /* Defer loading until applet is added to panel so
+   * gtk_widget_get_screen returns correct information */
+  g_signal_connect (GTK_WIDGET (applet), "realize",
+                    G_CALLBACK (on_applet_realized),
+                    NULL);
+
   atko = gtk_widget_get_accessible (GTK_WIDGET (applet));
   atk_object_set_name (atko, _("Panel Notification Area"));
 
   panel_applet_set_flags (applet,
                           PANEL_APPLET_HAS_HANDLE|PANEL_APPLET_EXPAND_MINOR);
   
-  g_signal_connect (applet, "change_orient",
-                    G_CALLBACK (applet_change_orientation), data);
-  g_signal_connect (applet, "change_background",
-                    G_CALLBACK (applet_change_background), data);
-  g_signal_connect (applet, "destroy",
-		    G_CALLBACK (applet_destroy), data);
-
   panel_applet_set_background_widget (applet, GTK_WIDGET (applet));
 
-  gtk_container_add (GTK_CONTAINER (applet), GTK_WIDGET (tray));
   force_no_focus_padding (GTK_WIDGET (applet));
 
 #ifndef NOTIFICATION_AREA_INPROCESS
   gtk_window_set_default_icon_name (NOTIFICATION_AREA_ICON);
 #endif
   gtk_widget_show_all (GTK_WIDGET (applet));
-
-  action_group = gtk_action_group_new ("ClockApplet Menu Actions");
-  gtk_action_group_set_translation_domain (action_group, GETTEXT_PACKAGE);
-  gtk_action_group_add_actions (action_group,
-				menu_actions,
-				G_N_ELEMENTS (menu_actions),
-				data);
-  ui_path = g_build_filename (NOTIFICATION_AREA_MENU_UI_DIR, "notification-area-menu.xml", NULL);
-  panel_applet_setup_menu_from_file (applet,
-				     ui_path, action_group);
-  g_free (ui_path);
-  g_object_unref (action_group);
-
   return TRUE;
 }
 
