@@ -8,8 +8,6 @@
  *
  */
 
-#define WNCK_I_KNOW_THIS_IS_UNSTABLE 1
-
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
@@ -213,25 +211,11 @@ applet_change_orient (PanelApplet       *applet,
 
 static void
 applet_change_background (PanelApplet               *applet,
-			  PanelAppletBackgroundType  type,
-			  GdkColor                  *color,
-			  GdkPixmap                 *pixmap,
+                          cairo_pattern_t           *pattern,
 			  PagerData                 *pager)
 {
-	switch (type) {
-	case PANEL_NO_BACKGROUND:
-		wnck_pager_set_shadow_type (WNCK_PAGER (pager->pager),
-					    GTK_SHADOW_IN);
-		break;
-	case PANEL_COLOR_BACKGROUND:
-		wnck_pager_set_shadow_type (WNCK_PAGER (pager->pager),
-					    GTK_SHADOW_NONE);
-		break;
-	case PANEL_PIXMAP_BACKGROUND:
-		wnck_pager_set_shadow_type (WNCK_PAGER (pager->pager),
-					    GTK_SHADOW_NONE);
-		break;
-	}
+        wnck_pager_set_shadow_type (WNCK_PAGER (pager->pager),
+                                    pattern != NULL ? GTK_SHADOW_NONE : GTK_SHADOW_IN);
 }
 
 static gboolean
@@ -533,7 +517,7 @@ workspace_switcher_applet_fill (PanelApplet *applet)
 		break;
 	}
 
-	pager->pager = wnck_pager_new (NULL);
+	pager->pager = wnck_pager_new ();
 	pager->screen = NULL;
 	pager->wm = PAGER_WM_UNKNOWN;
 	wnck_pager_set_shadow_type (WNCK_PAGER (pager->pager), GTK_SHADOW_IN);
@@ -731,10 +715,12 @@ static void
 num_workspaces_value_changed (GtkSpinButton *button,
 			      PagerData     *pager)
 {
+#if 0
 	/* Slow down a bit after the first change, since it's moving really to
 	 * fast. See bug #336731 for background.
 	 * FIXME: remove this if bug 410520 gets fixed. */
 	button->timer_step = 0.2;
+#endif
 
         wnck_screen_change_workspace_count (pager->screen,
                                             gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (pager->num_workspaces_spin)));
@@ -824,6 +810,8 @@ close_dialog (GtkWidget *button,
 {
 	PagerData *pager = data;
 	GtkTreeViewColumn *col;
+	GtkCellArea *area;
+	GtkCellEditable *edit_widget;
 
 	/* This is a hack. The "editable" signal for GtkCellRenderer is emitted
 	only on button press or focus cycle. Hence when the user changes the
@@ -832,9 +820,12 @@ close_dialog (GtkWidget *button,
 	to stop the editing. Thanks to Paolo for a better crack than the one I had.
 	*/
 
-	col = gtk_tree_view_get_column(GTK_TREE_VIEW (pager->workspaces_tree),0);
-	if (col->editable_widget != NULL && GTK_IS_CELL_EDITABLE (col->editable_widget))
-	    gtk_cell_editable_editing_done(col->editable_widget);
+	col = gtk_tree_view_get_column (GTK_TREE_VIEW (pager->workspaces_tree), 0);
+	area = gtk_cell_layout_get_area (GTK_CELL_LAYOUT (col));
+	edit_widget = gtk_cell_area_get_edit_widget (area);
+
+	if (edit_widget)
+		gtk_cell_editable_editing_done (edit_widget);
 
 	gtk_widget_destroy (pager->properties_dialog);
 }
