@@ -41,6 +41,8 @@ static GSList *registered_applets = NULL;
 static GSList *queued_position_saves = NULL;
 static guint   queued_position_source = 0;
 
+static void applet_menu_show       (GtkWidget *w, AppletInfo *info);
+static void applet_menu_deactivate (GtkWidget *w, AppletInfo *info);
 
 static inline PanelWidget *
 panel_applet_get_panel_widget (AppletInfo *info)
@@ -166,6 +168,9 @@ panel_applet_recreate_menu (AppletInfo	*info)
 		menu->submenu =NULL;
 	}
 
+        g_signal_handlers_disconnect_by_func (info->menu, G_CALLBACK (applet_menu_show), info);
+        g_signal_handlers_disconnect_by_func (info->menu, G_CALLBACK (applet_menu_deactivate), info);
+
 	g_object_unref (info->menu);
 	info->menu = panel_applet_create_menu (info);
 }
@@ -616,7 +621,7 @@ panel_applet_position_menu (GtkMenu   *menu,
 
 	screen = gtk_widget_get_screen (applet);
 
-	gtk_widget_size_request (GTK_WIDGET (menu), &requisition);
+	gtk_widget_get_preferred_size (GTK_WIDGET (menu), &requisition, NULL);
 
 	gdk_window_get_origin (gtk_widget_get_window (applet), &menu_x, &menu_y);
 	gtk_widget_get_pointer (applet, &pointer_x, &pointer_y);
@@ -773,8 +778,11 @@ panel_applet_destroy (GtkWidget  *widget,
 		panel_lockdown_notify_remove (G_CALLBACK (panel_applet_recreate_menu),
 					      info);
 
-	if (info->menu)
+	if (info->menu) {
+                g_signal_handlers_disconnect_by_func (info->menu, G_CALLBACK (applet_menu_show), info);
+                g_signal_handlers_disconnect_by_func (info->menu, G_CALLBACK (applet_menu_deactivate), info);
 		g_object_unref (info->menu);
+        }
 	info->menu = NULL;
 
 	if (info->data_destroy)
@@ -841,6 +849,11 @@ panel_applet_on_load_queue (const char *id)
 		if (strcmp (applet->id, id) == 0)
 			return TRUE;
 	}
+	for (li = panel_applets_loading; li != NULL; li = li->next) {
+		PanelAppletToLoad *applet = li->data;
+		if (strcmp (applet->id, id) == 0)
+			return TRUE;
+	}
 	return FALSE;
 }
 
@@ -908,7 +921,7 @@ panel_applet_load_idle_handler (gpointer dummy)
 	}
 
 	if (!l) {
-		/* All the rest of the applets don't have a panel */
+		/* All the remaining applets don't have a panel */
 		for (l = panel_applets_to_load; l; l = l->next)
 			free_applet_to_load (l->data);
 		g_slist_free (panel_applets_to_load);
@@ -1063,7 +1076,7 @@ panel_applet_load_queued_applets (gboolean initial_load)
 		return;
 	}
 
-	if (initial_load && panel_applets_to_load) {
+	if (panel_applets_to_load && panel_applet_unhide_toplevels_timeout == 0) {
 		/* Install a timeout to make sure we don't block the
 		 * unhiding because of an applet that doesn't load */
 		panel_applet_unhide_toplevels_timeout =
