@@ -50,7 +50,7 @@ enum {
 
 #define BUTTON_WIDGET_DISPLACEMENT 2
 
-G_DEFINE_TYPE (ButtonWidget, button_widget, GTK_TYPE_BUTTON);
+G_DEFINE_TYPE (ButtonWidget, button_widget, GTK_TYPE_BUTTON)
 
 /* colorshift a pixbuf */
 static void
@@ -113,47 +113,18 @@ make_hc_pixbuf (GdkPixbuf *pb)
 static void
 button_widget_realize(GtkWidget *widget)
 {
-	GdkWindowAttr attributes;
-	gint attributes_mask;
-	GtkButton *button;
+        // FIXMEchpe why?
+        gtk_widget_add_events (widget, GDK_POINTER_MOTION_MASK |
+                                       GDK_POINTER_MOTION_HINT_MASK |
+                                       GDK_KEY_PRESS_MASK);
 
-	g_return_if_fail (widget != NULL);
-	g_return_if_fail (BUTTON_IS_WIDGET (widget));
-
-	button = GTK_BUTTON (widget);
-
-	GTK_WIDGET_SET_FLAGS (widget, GTK_REALIZED);
-
-	attributes.window_type = GDK_WINDOW_CHILD;
-	attributes.x = widget->allocation.x;
-	attributes.y = widget->allocation.y;
-	attributes.width = widget->allocation.width;
-	attributes.height = widget->allocation.height;
-	attributes.wclass = GDK_INPUT_ONLY;
-	attributes.event_mask = (GDK_BUTTON_PRESS_MASK |
-				 GDK_BUTTON_RELEASE_MASK |
-				 GDK_POINTER_MOTION_MASK |
-				 GDK_POINTER_MOTION_HINT_MASK |
-				 GDK_KEY_PRESS_MASK |
-				 GDK_ENTER_NOTIFY_MASK |
-				 GDK_LEAVE_NOTIFY_MASK);
-	attributes_mask = GDK_WA_X | GDK_WA_Y;
-
-	widget->window = gtk_widget_get_parent_window (widget);
-	g_object_ref (G_OBJECT (widget->window));
-      
-	button->event_window = gdk_window_new (gtk_widget_get_parent_window (widget),
-					       &attributes,
-					       attributes_mask);
-	gdk_window_set_user_data (button->event_window, widget);
-
-	widget->style = gtk_style_attach (widget->style, widget->window);
+        GTK_WIDGET_CLASS (button_widget_parent_class)->realize (widget);
 
 	BUTTON_WIDGET (widget)->priv->icon_theme = gtk_icon_theme_get_for_screen (gtk_widget_get_screen (widget));
 	g_signal_connect_object (BUTTON_WIDGET (widget)->priv->icon_theme,
 				 "changed",
 				 G_CALLBACK (button_widget_icon_theme_changed),
-				 button,
+				 widget,
 				 G_CONNECT_SWAPPED);
 
 	button_widget_reload_pixbuf (BUTTON_WIDGET (widget));
@@ -162,20 +133,9 @@ button_widget_realize(GtkWidget *widget)
 static void
 button_widget_unrealize (GtkWidget *widget)
 {
-	GtkButton *button;
-	PanelWidget *panel;
-
-	g_return_if_fail (widget != NULL);
-	g_return_if_fail (BUTTON_IS_WIDGET (widget));
-
-	panel  = PANEL_WIDGET (widget->parent);
-	button = GTK_BUTTON (widget);
-
-	if (button->event_window != NULL) {
-		gdk_window_set_user_data (button->event_window, NULL);
-		gdk_window_destroy (button->event_window);
-		button->event_window = NULL;
-	}
+        g_signal_handlers_disconnect_by_func (BUTTON_WIDGET (widget)->priv->icon_theme,
+                                              G_CALLBACK (button_widget_icon_theme_changed),
+                                              widget);
 
 	GTK_WIDGET_CLASS (button_widget_parent_class)->unrealize (widget);
 }
@@ -214,11 +174,13 @@ button_widget_reload_pixbuf (ButtonWidget *button)
 		if (error) {
 			//FIXME: this is not rendered at button->priv->size
 			button->priv->pixbuf =
-				gtk_widget_render_icon (GTK_WIDGET (button),
-							GTK_STOCK_MISSING_IMAGE,
-							(GtkIconSize) -1, NULL);
+				gtk_widget_render_icon_pixbuf (GTK_WIDGET (button),
+                                                               GTK_STOCK_MISSING_IMAGE,
+                                                               (GtkIconSize) -1);
 			g_free (error);
 		}
+
+                gtk_button_set_image (GTK_BUTTON (button), gtk_image_new_from_pixbuf (button->priv->pixbuf));
 	}
 
 	button->priv->pixbuf_hc = make_hc_pixbuf (button->priv->pixbuf);
@@ -254,8 +216,6 @@ button_widget_get_property (GObject    *object,
 {
 	ButtonWidget *button;
 
-	g_return_if_fail (BUTTON_IS_WIDGET (object));
-
 	button = BUTTON_WIDGET (object);
 
 	switch (prop_id) {
@@ -291,8 +251,6 @@ button_widget_set_property (GObject      *object,
 {
 	ButtonWidget *button;
 
-	g_return_if_fail (BUTTON_IS_WIDGET (object));
-
 	button = BUTTON_WIDGET (object);
 
 	switch (prop_id) {
@@ -326,36 +284,40 @@ calc_arrow (PanelOrientation  orientation,
 	    int               button_height,
 	    int              *x,
 	    int              *y,
-	    int              *width,
-	    int              *height)
+            gdouble          *angle,
+	    gdouble          *size)
 {
 	GtkArrowType retval = GTK_ARROW_UP;
 	double scale;
 
 	scale = (orientation & PANEL_HORIZONTAL_MASK ? button_height : button_width) / 48.0;
 
-	*width  = 12 * scale;
-	*height = 12 * scale;
+        *size = 12 * scale;
+        *angle = 0;
 
 	switch (orientation) {
 	case PANEL_ORIENTATION_TOP:
 		*x     = scale * 3;
 		*y     = scale * (48 - 13);
+                *angle = G_PI;
 		retval = GTK_ARROW_DOWN;
 		break;
 	case PANEL_ORIENTATION_BOTTOM:
 		*x     = scale * (48 - 13);
 		*y     = scale * 1;
+                *angle = 0;
 		retval = GTK_ARROW_UP;
 		break;
 	case PANEL_ORIENTATION_LEFT:
 		*x     = scale * (48 - 13);
 		*y     = scale * 3;
+                *angle = G_PI / 2;
 		retval = GTK_ARROW_RIGHT;
 		break;
 	case PANEL_ORIENTATION_RIGHT:
 		*x     = scale * 1;
 		*y     = scale * 3;
+                *angle = 3 * (G_PI / 2);
 		retval = GTK_ARROW_LEFT;
 		break;
 	}
@@ -364,33 +326,30 @@ calc_arrow (PanelOrientation  orientation,
 }
 
 static gboolean
-button_widget_expose (GtkWidget         *widget,
-		      GdkEventExpose    *event)
+button_widget_draw (GtkWidget *widget,
+		    cairo_t   *cr)
 {
-	ButtonWidget *button_widget;
-	GtkButton *button;
-	GdkRectangle area, image_bound;
+	ButtonWidget *button_widget = BUTTON_WIDGET (widget);
+	GtkStyleContext *context;
+	GtkStateFlags state_flags;
 	int off;
-	int x, y, w, h;
+        int width, height;
+        int x, y, w, h;
 	GdkPixbuf *pb = NULL;
-  
-	g_return_val_if_fail (BUTTON_IS_WIDGET (widget), FALSE);
-	g_return_val_if_fail (event != NULL, FALSE);
-
-	button_widget = BUTTON_WIDGET (widget);
-	button = GTK_BUTTON (widget);
-	
-	if (!GTK_WIDGET_VISIBLE (widget) || !GTK_WIDGET_MAPPED (widget))
-		return FALSE;
 
 	if (!button_widget->priv->pixbuf_hc && !button_widget->priv->pixbuf)
 		return FALSE;
 
-	/* offset for pressed buttons */
-	off = (button_widget->priv->activatable &&
-	       button->in_button && button->button_down) ?
-		BUTTON_WIDGET_DISPLACEMENT * widget->allocation.height / 48.0 : 0;
+	width = gtk_widget_get_allocated_width (widget);
+        height = gtk_widget_get_allocated_height (widget);
 
+	/* offset for pressed buttons */
+	state_flags = gtk_widget_get_state_flags (widget);
+	off = (button_widget->priv->activatable && (state_flags & GTK_STATE_FLAG_PRELIGHT) &&
+	       (state_flags & GTK_STATE_FLAG_ACTIVE)) ?
+		BUTTON_WIDGET_DISPLACEMENT * height / 48.0 : 0;
+
+        /* FIXMEchpe replace this by cairo ops too! */
 	if (!button_widget->priv->activatable) {
 		pb = gdk_pixbuf_copy (button_widget->priv->pixbuf);
 		gdk_pixbuf_saturate_and_pixelate (button_widget->priv->pixbuf,
@@ -398,7 +357,7 @@ button_widget_expose (GtkWidget         *widget,
 						  0.8,
 						  TRUE);
 	} else if (panel_global_config_get_highlight_when_over () && 
-		   (button->in_button || GTK_WIDGET_HAS_FOCUS (widget)))
+		   (state_flags & GTK_STATE_FLAG_PRELIGHT || gtk_widget_has_focus (widget)))
 		pb = g_object_ref (button_widget->priv->pixbuf_hc);
 	else
 		pb = g_object_ref (button_widget->priv->pixbuf);
@@ -407,104 +366,118 @@ button_widget_expose (GtkWidget         *widget,
 
 	w = gdk_pixbuf_get_width (pb);
 	h = gdk_pixbuf_get_height (pb);
-	x = widget->allocation.x + off + (widget->allocation.width - w)/2;
-	y = widget->allocation.y + off + (widget->allocation.height - h)/2;
-	
-	image_bound.x = x;
-	image_bound.y = y;      
-	image_bound.width = w;
-	image_bound.height = h;
-	
-	area = event->area;
-	
-	if (gdk_rectangle_intersect (&area, &widget->allocation, &area) &&
-	    gdk_rectangle_intersect (&image_bound, &area, &image_bound))
-		gdk_draw_pixbuf (widget->window, NULL, pb,
-				 image_bound.x - x, image_bound.y - y,
-				 image_bound.x, image_bound.y,
-				 image_bound.width, image_bound.height,
-				 GDK_RGB_DITHER_NORMAL,
-				 0, 0);
+	x = off + (width - w)/2;
+	y = off + (height - h)/2;
+
+        cairo_save (cr);
+        gdk_cairo_set_source_pixbuf (cr, pb, x, y);
+        cairo_paint (cr);
+        cairo_restore (cr);
 
 	g_object_unref (pb);
-	
+
+        context = gtk_widget_get_style_context (widget);
+
 	if (button_widget->priv->arrow) {
 		GtkArrowType arrow_type;
-		int          x, y, width, height;
+                gdouble angle, size;
 
-		x = y = width = height = -1;
+                gtk_style_context_save (context);
+                gtk_style_context_set_state (context, state_flags);
 
 		arrow_type = calc_arrow (button_widget->priv->orientation,
-					 widget->allocation.width,
-					 widget->allocation.height,
-					 &x,
-					 &y,
-					 &width,
-					 &height);
+					 width, height,
+					 &x, &y,
+                                         &angle, &size);
 
-		gtk_paint_arrow (widget->style,
-				 widget->window,
-				 GTK_STATE_NORMAL,
-				 GTK_SHADOW_NONE,
-				 NULL,
-				 widget,
-				 "panel-button",
-				 arrow_type,
-				 TRUE,
-				 widget->allocation.x + x,
-				 widget->allocation.y + y,
-				 width,
-				 height);
+                cairo_save (cr);
+		gtk_render_arrow (context, cr, angle, x, y, size);
+                cairo_restore (cr);
+
+                gtk_style_context_restore (context);
 	}
 
 	if (button_widget->priv->dnd_highlight) {
-		gdk_draw_rectangle(widget->window, widget->style->black_gc, FALSE,
-				   widget->allocation.x, widget->allocation.y,
-				   widget->allocation.width - 1,
-				   widget->allocation.height - 1);
+                cairo_save (cr);
+                cairo_set_line_width (cr, 1);
+                cairo_set_source_rgb (cr, 0., 0., 0.);
+                cairo_rectangle (cr, 0.5, 0.5, width - 1, height - 1);
+                cairo_stroke (cr);
+                cairo_restore (cr);
 	}
 
-	if (GTK_WIDGET_HAS_FOCUS (widget)) {
-		gint focus_width, focus_pad;
-		gint x, y, width, height;
+	if (gtk_widget_has_focus (widget)) {
+		gint focus_pad;
+
+                gtk_style_context_save (context);
+                gtk_style_context_set_state (context, state_flags);
 
 		gtk_widget_style_get (widget,
-				      "focus-line-width", &focus_width,
 				      "focus-padding", &focus_pad,
 				      NULL);
-		x = widget->allocation.x + focus_pad;
-		y = widget->allocation.y + focus_pad;
-		width = widget->allocation.width -  2 * focus_pad;
-		height = widget->allocation.height - 2 * focus_pad;
-		gtk_paint_focus (widget->style, widget->window,
-				 GTK_STATE_NORMAL,
-				 &event->area, widget, "button",
-				 x, y, width, height);
+		x = focus_pad;
+		y = focus_pad;
+		w = width -  2 * focus_pad;
+		h = height - 2 * focus_pad;
+
+                cairo_save (cr);
+		gtk_render_focus (context, cr, x, y, w, h);
+                cairo_restore (cr);
+
+                gtk_style_context_restore (context);
 	}
 	
 	return FALSE;
 }
 
 static void
-button_widget_size_request (GtkWidget      *widget,
-			    GtkRequisition *requisition)
+button_widget_get_preferred_width (GtkWidget *widget,
+				   gint *minimal_width,
+				   gint *natural_width)
 {
 	ButtonWidget *button_widget = BUTTON_WIDGET (widget);
+	GtkWidget *parent;
+	int size;
 
-	if (button_widget->priv->pixbuf) {
-		requisition->width  = gdk_pixbuf_get_width  (button_widget->priv->pixbuf);
-		requisition->height = gdk_pixbuf_get_height (button_widget->priv->pixbuf);
-	}
+	parent = gtk_widget_get_parent (widget);
+
+	if (button_widget->priv->orientation & PANEL_HORIZONTAL_MASK)
+		size = gtk_widget_get_allocated_height (parent);
+	else
+		size = gtk_widget_get_allocated_width (parent);
+
+	*minimal_width = *natural_width = size;
 }
 
+static void
+button_widget_get_preferred_height (GtkWidget *widget,
+				    gint *minimal_height,
+				    gint *natural_height)
+{
+	ButtonWidget *button_widget = BUTTON_WIDGET (widget);
+	GtkWidget *parent;
+	int size;
+
+	parent = gtk_widget_get_parent (widget);
+
+	if (button_widget->priv->orientation & PANEL_HORIZONTAL_MASK)
+		size = gtk_widget_get_allocated_height (parent);
+	else
+		size = gtk_widget_get_allocated_width (parent);
+
+	*minimal_height = *natural_height = size;
+}
+
+
 static void
 button_widget_size_allocate (GtkWidget     *widget,
 			     GtkAllocation *allocation)
 {
 	ButtonWidget *button_widget = BUTTON_WIDGET (widget);
-	GtkButton    *button = GTK_BUTTON (widget);
 	int           size;
 
+        GTK_WIDGET_CLASS (button_widget_parent_class)->size_allocate (widget, allocation);
+
 	if (button_widget->priv->orientation & PANEL_HORIZONTAL_MASK)
 		size = allocation->height;
 	else
@@ -526,16 +499,6 @@ button_widget_size_allocate (GtkWidget     *widget,
 
 		button_widget_reload_pixbuf (button_widget);
 	}
-
-	widget->allocation = *allocation;
-
-	if (GTK_WIDGET_REALIZED (widget)) {
-		gdk_window_move_resize (button->event_window, 
-					allocation->x,
-					allocation->y,
-					allocation->width,
-					allocation->height);
-	}
 }
 
 static void
@@ -568,13 +531,18 @@ button_widget_button_press (GtkWidget *widget, GdkEventButton *event)
 static gboolean
 button_widget_enter_notify (GtkWidget *widget, GdkEventCrossing *event)
 {
+	GtkStateFlags state_flags;
 	gboolean in_button;
 
 	g_return_val_if_fail (BUTTON_IS_WIDGET (widget), FALSE);
 
-	in_button = GTK_BUTTON (widget)->in_button;
+	state_flags = gtk_widget_get_state_flags (widget);
+	in_button = state_flags & GTK_STATE_FLAG_PRELIGHT;
+
 	GTK_WIDGET_CLASS (button_widget_parent_class)->enter_notify_event (widget, event);
-	if (in_button != GTK_BUTTON (widget)->in_button &&
+
+	state_flags = gtk_widget_get_state_flags (widget);
+	if (in_button != (state_flags & GTK_STATE_FLAG_PRELIGHT) &&
 	    panel_global_config_get_highlight_when_over ())
 		gtk_widget_queue_draw (widget);
 
@@ -584,13 +552,18 @@ button_widget_enter_notify (GtkWidget *widget, GdkEventCrossing *event)
 static gboolean
 button_widget_leave_notify (GtkWidget *widget, GdkEventCrossing *event)
 {
+	GtkStateFlags state_flags;
 	gboolean in_button;
 
 	g_return_val_if_fail (BUTTON_IS_WIDGET (widget), FALSE);
 
-	in_button = GTK_BUTTON (widget)->in_button;
+	state_flags = gtk_widget_get_state_flags (widget);
+	in_button = state_flags & GTK_STATE_FLAG_PRELIGHT;
+
 	GTK_WIDGET_CLASS (button_widget_parent_class)->leave_notify_event (widget, event);
-	if (in_button != GTK_BUTTON (widget)->in_button &&
+
+	state_flags = gtk_widget_get_state_flags (widget);
+	if (in_button != (state_flags & GTK_STATE_FLAG_PRELIGHT) &&
 	    panel_global_config_get_highlight_when_over ())
 		gtk_widget_queue_draw (widget);
 
@@ -631,14 +604,15 @@ button_widget_class_init (ButtonWidgetClass *klass)
 
 	g_type_class_add_private (klass, sizeof (ButtonWidgetPrivate));
 	  
-	widget_class->realize            = button_widget_realize;
-	widget_class->unrealize          = button_widget_unrealize;
-	widget_class->size_allocate      = button_widget_size_allocate;
-	widget_class->size_request       = button_widget_size_request;
-	widget_class->button_press_event = button_widget_button_press;
-	widget_class->enter_notify_event = button_widget_enter_notify;
-	widget_class->leave_notify_event = button_widget_leave_notify;
-	widget_class->expose_event       = button_widget_expose;
+	widget_class->realize              = button_widget_realize;
+	widget_class->unrealize            = button_widget_unrealize;
+	widget_class->size_allocate        = button_widget_size_allocate;
+	widget_class->get_preferred_width  = button_widget_get_preferred_width;
+	widget_class->get_preferred_height = button_widget_get_preferred_height;
+	widget_class->button_press_event   = button_widget_button_press;
+	widget_class->enter_notify_event   = button_widget_enter_notify;
+	widget_class->leave_notify_event   = button_widget_leave_notify;
+	widget_class->draw                 = button_widget_draw;
 
 	button_class->activate = button_widget_activate;
 
@@ -726,7 +700,7 @@ button_widget_set_activatable (ButtonWidget *button,
 	if (button->priv->activatable != activatable) {
 		button->priv->activatable = activatable;
 
-		if (GTK_WIDGET_DRAWABLE (button))
+		if (gtk_widget_is_drawable (GTK_WIDGET (button)))
 			gtk_widget_queue_draw (GTK_WIDGET (button));
 
 		g_object_notify (G_OBJECT (button), "activatable");
@@ -876,7 +850,7 @@ button_widget_get_ignore_leave (ButtonWidget *button)
 GtkIconTheme *
 button_widget_get_icon_theme (ButtonWidget *button)
 {
-	g_return_val_if_fail (BUTTON_IS_WIDGET (button), FALSE);
+	g_return_val_if_fail (BUTTON_IS_WIDGET (button), NULL);
 
 	return button->priv->icon_theme;
 }
@@ -884,7 +858,7 @@ button_widget_get_icon_theme (ButtonWidget *button)
 GdkPixbuf *
 button_widget_get_pixbuf (ButtonWidget *button)
 {
-	g_return_val_if_fail (BUTTON_IS_WIDGET (button), FALSE);
+	g_return_val_if_fail (BUTTON_IS_WIDGET (button), NULL);
 
 	if (!button->priv->pixbuf)
 		return NULL;
