@@ -20,12 +20,13 @@
 
 #include <glib/gi18n.h>
 #include <gio/gio.h>
-#include <libgnome/gnome-util.h>
-#include <libgnomeui/gnome-url.h>
 #include <gdk/gdkx.h>
 
+#include <libpanel-util/panel-error.h>
 #include <libpanel-util/panel-glib.h>
 #include <libpanel-util/panel-keyfile.h>
+#include <libpanel-util/panel-launch.h>
+#include <libpanel-util/panel-show.h>
 
 #include "launcher.h"
 
@@ -53,7 +54,7 @@ launcher_get_screen (Launcher *launcher)
 	g_return_val_if_fail (launcher->info != NULL, NULL);
 	g_return_val_if_fail (launcher->info->widget != NULL, NULL);
 
-	panel_widget = PANEL_WIDGET (launcher->info->widget->parent);
+	panel_widget = PANEL_WIDGET (gtk_widget_get_parent (launcher->info->widget));
 
 	return gtk_window_get_screen (GTK_WINDOW (panel_widget->toplevel));
 }
@@ -95,66 +96,10 @@ launcher_register_error_dialog (Launcher *launcher,
 			  launcher);
 }
 
-static GFile *
-panel_launcher_get_gfile (const char *location)
-{
-	char  *path;
-	GFile *file;
-
-	if (!g_ascii_strncasecmp (location, "file:", strlen ("file:")))
-		return g_file_new_for_uri (location);
-
-	if (g_path_is_absolute (location))
-		return g_file_new_for_path (location);
-
-	path = panel_make_full_path (NULL, location);
-	file = g_file_new_for_path (path);
-	g_free (path);
-
-	return file;
-}
-
-static char *
-panel_launcher_get_uri (const char *location)
-{
-	char *path;
-	char *uri;
-
-	if (!g_ascii_strncasecmp (location, "file:", strlen ("file:")))
-		return g_strdup (location);
-
-	if (!g_path_is_absolute (location))
-		path = panel_make_full_path (NULL, location);
-	else
-		path = g_strdup (location);
-
-	uri = g_filename_to_uri (path, NULL, NULL);
-	g_free (path);
-
-	return uri;
-}
-
-static const char *
-panel_launcher_get_filename (const char *location)
-{
-	char *p;
-
-	if (!g_path_is_absolute (location) &&
-	    g_ascii_strncasecmp (location, "file:", strlen ("file:")))
-		/* this is not a local URI */
-		return NULL;
-
-	if ((p = strstr (location, PANEL_LAUNCHERS_PATH)))
-		p += sizeof (PANEL_LAUNCHERS_PATH);
-
-	return p;
-}
-
 static void
 launch_url (Launcher *launcher)
 {
 	char *url;
-	GError *error = NULL;
 	GdkScreen *screen;
 
 	g_return_if_fail (launcher != NULL);
@@ -177,21 +122,7 @@ launch_url (Launcher *launcher)
 		return;
 	}
 
-	gnome_url_show_on_screen (url, screen, &error);
-
-	if (error) {
-		GtkWidget *error_dialog;
-		char      *primary;
-	
-		primary = g_strdup_printf (_("Could not show '%s'"), url);
-		error_dialog = panel_error_dialog (NULL, screen,
-						   "cannot_show_url_dialog",
-						   TRUE,
-						   primary, error->message);
-		g_free (primary);
-		launcher_register_error_dialog (launcher, error_dialog);
-		g_clear_error (&error);
-	}
+	panel_show_uri (screen, url, gtk_get_current_event_time (), NULL);
 
 	g_free (url);
 }
@@ -217,10 +148,8 @@ launcher_launch (Launcher  *launcher,
 	else {
 		GError *error = NULL;
 
-		panel_util_launch_from_key_file (launcher->key_file,
-						 NULL,
-						 launcher_get_screen (launcher),
-						 &error);
+		panel_launch_key_file (launcher->key_file, NULL,
+				       launcher_get_screen (launcher), &error);
 		if (error) {
 			GtkWidget *error_dialog;
 
@@ -241,7 +170,7 @@ launcher_launch (Launcher  *launcher,
 		PanelToplevel *toplevel;
 		PanelToplevel *parent;
 
-		toplevel = PANEL_WIDGET (launcher->button->parent)->toplevel;
+		toplevel = PANEL_WIDGET (gtk_widget_get_parent (launcher->button))->toplevel;
 
 		if (panel_toplevel_get_is_attached (toplevel)) {
 			parent = panel_toplevel_get_attach_toplevel (toplevel);
@@ -278,15 +207,13 @@ drag_data_received_cb (GtkWidget        *widget,
 				     NULL);
 	
 	file_list = NULL;
-	uris = g_uri_list_extract_uris ((const char *) selection_data->data);
+	uris = g_uri_list_extract_uris ((const char *) gtk_selection_data_get_data (selection_data));
 	for (i = 0; uris[i]; i++)
 		file_list = g_list_prepend (file_list, uris[i]);
 	file_list = g_list_reverse (file_list);
 
-	panel_util_launch_from_key_file (launcher->key_file,
-					 file_list,
-					 launcher_get_screen (launcher),
-					 &error);
+	panel_launch_key_file (launcher->key_file, file_list,
+			       launcher_get_screen (launcher), &error);
 
 	g_list_free (file_list);
 	g_strfreev (uris);
@@ -345,22 +272,15 @@ free_launcher (gpointer data)
 void
 panel_launcher_delete (Launcher *launcher)
 {
-	GFile *file;
-	GFile *launchers;
-	char  *launchers_path;
-
 	if (!launcher->location)
 		return;
 
-	launchers_path = gnome_util_home_file (PANEL_LAUNCHERS_PATH);
-	launchers = g_file_new_for_path (launchers_path);
-	g_free (launchers_path);
-
-	file = panel_launcher_get_gfile (launcher->location);
-
 	/* do not remove the file if it's not in the user's launchers path */
-	if (g_file_has_prefix (file, launchers)) {
+	if (panel_launcher_is_in_personal_path (launcher->location)) {
 		GError *error;
+		GFile  *file;
+
+		file = panel_launcher_get_gfile (launcher->location);
 
 		error = NULL;
 		if (!g_file_delete (file, NULL, &error)) {
@@ -372,10 +292,9 @@ panel_launcher_delete (Launcher *launcher)
 			g_free (path);
 			g_error_free (error);
 		}
-	}
 
-	g_object_unref (file);
-	g_object_unref (launchers);
+		g_object_unref (file);
+	}
 }
 
 static gboolean
@@ -391,13 +310,13 @@ is_this_drop_ok (GtkWidget      *widget,
 	if (source == widget)
 		return FALSE;
 
-	if (!(context->actions & GDK_ACTION_COPY))
+	if (!(gdk_drag_context_get_actions (context) & GDK_ACTION_COPY))
 		return FALSE;
 
 	if (!text_uri_list)
 		text_uri_list = gdk_atom_intern_static_string ("text/uri-list");
 
-	for (l = context->targets; l; l = l->next) {
+	for (l = gdk_drag_context_list_targets (context); l; l = l->next) {
 		if (GDK_POINTER_TO_ATOM (l->data) == text_uri_list)
 			break;
 	}
@@ -485,7 +404,7 @@ drag_data_get_cb (GtkWidget        *widget,
 		g_free (uri[0]);
 	} else if (info == TARGET_ICON_INTERNAL)
 		gtk_selection_data_set (selection_data,
-					selection_data->target, 8,
+					gtk_selection_data_get_target (selection_data), 8,
 					(unsigned char *) location,
 					strlen (location));
 
@@ -516,12 +435,8 @@ create_launcher (const char *location)
 		path = panel_make_full_path (NULL, location);
 
 		if (!g_file_test (path, G_FILE_TEST_EXISTS)) {
-			char *buffer;
-
 			g_free (path);
-			buffer = g_strconcat ("applications/", location, NULL);
-			path = panel_lookup_in_data_dirs (buffer);
-			g_free (buffer);
+			path = panel_g_lookup_in_applications_dirs (location);
 			/* it's important to keep the full path if the desktop
 			 * file comes from a data dir: when the user will edit
 			 * it, we'll want to save it in PANEL_LAUNCHERS_PATH
@@ -820,7 +735,7 @@ launcher_properties (Launcher  *launcher)
 							_("Launcher Properties"));
 
 	panel_widget_register_open_dialog (PANEL_WIDGET 
-					   (launcher->info->widget->parent),
+					   (gtk_widget_get_parent (launcher->info->widget)),
 					   launcher->prop_dialog);
 
 	panel_ditem_register_save_uri_func (PANEL_DITEM_EDITOR (launcher->prop_dialog),
@@ -1040,6 +955,8 @@ panel_launcher_create_from_info (PanelToplevel *toplevel,
 	}
 
 	location = panel_make_unique_desktop_uri (NULL, exec_or_uri);
+
+	error = NULL;
 	if (panel_key_file_to_file (key_file, location, &error)) {
 		panel_launcher_create (toplevel, position, location);
 	} else {
@@ -1177,7 +1094,7 @@ panel_launcher_set_dnd_enabled (Launcher *launcher,
 			{ "text/uri-list", 0, TARGET_URI_LIST }
 		};
 
-		GTK_WIDGET_UNSET_FLAGS (launcher->button, GTK_NO_WINDOW);
+		gtk_widget_set_has_window (launcher->button, TRUE);
 		gtk_drag_source_set (launcher->button,
 				     GDK_BUTTON1_MASK,
 				     dnd_targets, 2,
@@ -1189,7 +1106,7 @@ panel_launcher_set_dnd_enabled (Launcher *launcher,
 							 pixbuf);
 			g_object_unref (pixbuf);
 		}
-		GTK_WIDGET_SET_FLAGS (launcher->button, GTK_NO_WINDOW);
+		gtk_widget_set_has_window (launcher->button, FALSE);
 	
 
 	} else
