@@ -36,6 +36,7 @@
 #include <libpanel-util/panel-show.h>
 
 #include "applet.h"
+#include "panel-compatibility.h"
 #include "panel-widget.h"
 #include "panel-util.h"
 #include "panel-profile.h"
@@ -64,6 +65,7 @@ typedef enum {
 	APPLICATIONS_MENU,
 #define DEFAULT_MENU      APPLICATIONS_MENU
 	SETTINGS_MENU,
+	GNOMECC_MENU,
 	LAST_MENU
 } MenuPathRoot;
 
@@ -75,7 +77,9 @@ typedef struct {
 
 static MenuPathRootItem root_items [] = {
 	{ APPLICATIONS_MENU, "applications", "applications.menu" },
-	{ SETTINGS_MENU,     "settings",     "settings.menu"     }
+	{ GNOMECC_MENU,      "gnomecc",      "gnomecc.menu"      },
+	/* compatibility with GNOME < 2.91.0 */
+	{ SETTINGS_MENU,     "settings",     NULL                }
 };
 
 struct _PanelMenuButtonPrivate {
@@ -349,7 +353,9 @@ panel_menu_button_menu_deactivated (PanelMenuButton *button)
 {
 	panel_toplevel_pop_autohide_disabler (button->priv->toplevel);
 
-	GTK_BUTTON (button)->in_button = FALSE;
+	gtk_widget_unset_state_flags (GTK_WIDGET (button),
+				      GTK_STATE_FLAG_PRELIGHT);
+
 	button_widget_set_ignore_leave (BUTTON_WIDGET (button), FALSE);
 }
 
@@ -870,19 +876,30 @@ panel_menu_button_load_from_gconf (PanelWidget *panel,
 				   gboolean     exactpos,
 				   const char  *id)
 {
-	GConfClient *client;
-	const char  *key;
-	char        *menu_path;
-	char        *custom_icon;
-	char        *tooltip;
-	gboolean     use_menu_path;
-	gboolean     use_custom_icon;
+	GConfClient  *client;
+	char         *scheme;
+	MenuPathRoot  root;
+	const char   *key;
+	char         *menu_path;
+	char         *custom_icon;
+	char         *tooltip;
+	gboolean      use_menu_path;
+	gboolean      use_custom_icon;
 
 	client  = panel_gconf_get_client ();
 
 	key = panel_gconf_full_key (PANEL_GCONF_OBJECTS, id, "menu_path");
 	menu_path = gconf_client_get_string (client, key, NULL);
 
+	scheme = g_strndup (menu_path, strcspn (menu_path, ":"));
+	root = panel_menu_scheme_to_path_root (scheme);
+	g_free (scheme);
+	if (root == SETTINGS_MENU) {
+		panel_compatiblity_migrate_settings_menu_button (client, id);
+		g_free (menu_path);
+		return;
+       }
+
 	key = panel_gconf_full_key (PANEL_GCONF_OBJECTS, id, "custom_icon");
 	custom_icon = gconf_client_get_string (client, key, NULL);
 
