@@ -139,7 +139,7 @@ activate_desktop_uri (GtkWidget *menuitem,
 	activate_path (menuitem,
 		       g_get_user_special_dir (G_USER_DIRECTORY_DESKTOP));
 }
- 
+
 static void
 panel_menu_items_append_from_desktop (GtkWidget *menu,
 				      char      *path,
@@ -1074,20 +1074,29 @@ panel_place_menu_item_create_menu (PanelPlaceMenuItem *place_item)
 	gconf_name = gconf_client_get_string (panel_gconf_get_client (),
 					      COMPUTER_NAME_KEY,
 					      NULL);
-	panel_menu_items_append_from_desktop (places_menu,
-					      "nautilus-computer.desktop",
-					      gconf_name,
-                                              TRUE);
-	if (gconf_name)
-		g_free (gconf_name);
+
+	if (gconf_name == NULL) {
+		gconf_name = g_strdup (_("Computer"));
+	}
+
+	panel_menu_items_append_place_item (PANEL_ICON_COMPUTER, NULL,
+					    gconf_name,
+					    _("Browse all local and remote disks and folders accessible from this computer"),
+					    places_menu,
+					    G_CALLBACK (activate_uri),
+					    "computer://");
+	g_free (gconf_name);
 
 	panel_place_menu_item_append_local_gio (place_item, places_menu);
 	add_menu_separator (places_menu);
 
-	panel_menu_items_append_from_desktop (places_menu,
-					      "network-scheme.desktop",
-					      NULL,
-                                              TRUE);
+	panel_menu_items_append_place_item (PANEL_ICON_NETWORK, NULL,
+					    _("Network"),
+					    _("Browse bookmarked and local network locations"),
+					    places_menu,
+					    G_CALLBACK (activate_uri),
+					    "network://");
+
 	panel_place_menu_item_append_remote_gio (place_item, places_menu);
 
 	if (panel_is_program_in_path ("nautilus-connect-server")) {
@@ -1171,49 +1180,22 @@ panel_place_menu_item_mounts_changed (GVolumeMonitor *monitor,
 	panel_place_menu_item_recreate_menu (place_menu);
 }
 
-static void
-panel_desktop_menu_item_append_menu (GtkWidget *menu,
-				     gpointer   data)
-{
-	PanelDesktopMenuItem *parent;
-	gboolean              add_separator;
-	GList                *children;
-	GList                *last;
-
-	parent = PANEL_DESKTOP_MENU_ITEM (data);
-
-	add_separator = FALSE;
-	children = gtk_container_get_children (GTK_CONTAINER (menu));
-	last = g_list_last (children);
-
-	if (last != NULL)
-		add_separator = !GTK_IS_SEPARATOR (GTK_WIDGET (last->data));
-
-	g_list_free (children);
-
-	if (add_separator)
-		add_menu_separator (menu);
-
-	panel_menu_items_append_from_desktop (menu, "yelp.desktop", NULL, FALSE);
-	panel_menu_items_append_from_desktop (menu, "gnome-about.desktop", NULL, FALSE);
-
-	if (parent->priv->append_lock_logout)
-		panel_menu_items_append_lock_logout (menu);
-}
-
 static GtkWidget *
 panel_desktop_menu_item_create_menu (PanelDesktopMenuItem *desktop_item)
 {
 	GtkWidget *desktop_menu;
 
-	desktop_menu = create_applications_menu ("settings.menu", NULL, FALSE);
+	desktop_menu = panel_create_menu ();
 
-	g_object_set_data (G_OBJECT (desktop_menu),
-			   "panel-menu-append-callback",
-			   panel_desktop_menu_item_append_menu);
-	g_object_set_data (G_OBJECT (desktop_menu),
-			   "panel-menu-append-callback-data",
-			   desktop_item);
+	panel_menu_items_append_from_desktop (desktop_menu,
+					      "gnome-control-center.desktop",
+					      NULL, FALSE);
+	panel_menu_items_append_from_desktop (desktop_menu,
+					      "yelp.desktop",
+					      NULL, FALSE);
+
+	if (desktop_item->priv->append_lock_logout)
+		panel_menu_items_append_lock_logout (desktop_menu);
 
 	return desktop_menu;
 }
