@@ -188,7 +188,7 @@ panel_recent_append_documents_menu (GtkWidget        *top_menu,
 	GtkWidget      *menu_item;
 	int             size;
 
-	menu_item = panel_image_menu_item_new ();
+	menu_item = gtk_image_menu_item_new ();
 	setup_menu_item_with_icon (menu_item,
 				   panel_menu_icon_get_size (),
 				   PANEL_ICON_RECENT,
@@ -227,12 +227,12 @@ panel_recent_append_documents_menu (GtkWidget        *top_menu,
 
 	add_menu_separator (recent_menu);
 
-	menu_item = panel_image_menu_item_new ();
+	menu_item = gtk_image_menu_item_new ();
 	setup_menu_item_with_icon (menu_item,
 				   panel_menu_icon_get_size (),
 				   NULL,
 				   GTK_STOCK_CLEAR, NULL,
-				   _("Clear Recent Documents"));
+				   _("Clear Recent Documents..."));
 	panel_util_set_tooltip_text (menu_item,
 				     _("Clear all items from the recent documents list"));
 	gtk_menu_shell_append (GTK_MENU_SHELL (recent_menu), menu_item);
