@@ -110,7 +110,7 @@ panel_push_window_busy (GtkWidget *window)
 		if (win != NULL) {
 			GdkCursor *cursor = gdk_cursor_new (GDK_WATCH);
 			gdk_window_set_cursor (win, cursor);
-			gdk_cursor_unref (cursor);
+			g_object_unref (cursor);
 			gdk_flush ();
 		}
 	}
@@ -399,6 +399,9 @@ panel_lock_screen_action (GdkScreen  *screen,
 {
 	GError  *error            = NULL;
 	char    *command          = NULL;
+	GdkAppLaunchContext *launch_context = NULL;
+	GAppInfo            *app_info = NULL;
+	GdkDisplay          *display;
 
 	g_return_if_fail (GDK_IS_SCREEN (screen));
 	g_return_if_fail (action != NULL);
@@ -412,7 +415,22 @@ panel_lock_screen_action (GdkScreen  *screen,
 	if (!command)
 		return;
 
-	if (!gdk_spawn_command_line_on_screen (screen, command, &error)) {
+	app_info = g_app_info_create_from_commandline (command,
+						       NULL,
+						       G_APP_INFO_CREATE_NONE,
+						       &error);
+
+	if (!error) {
+		display = gdk_screen_get_display (screen);
+		launch_context = gdk_display_get_app_launch_context (display);
+		gdk_app_launch_context_set_screen (launch_context, screen);
+		g_app_info_launch (app_info, NULL, G_APP_LAUNCH_CONTEXT (launch_context), &error);
+
+		g_object_unref (launch_context);
+	}
+
+	/* error either by create or launch the command */
+	if (error) {
 		char *primary;
 
 		primary = g_strdup_printf (_("Could not execute '%s'"),
@@ -425,6 +443,7 @@ panel_lock_screen_action (GdkScreen  *screen,
 	}
 
 	g_free (command);
+	g_object_unref (app_info);
 }
 
 void
@@ -439,8 +458,16 @@ panel_lock_screen (GdkScreen *screen)
 static char *
 panel_launcher_get_personal_path (void)
 {
-	return g_build_filename (g_get_home_dir (), ".gnome2",
-				 PANEL_LAUNCHER_PERSONAL_PATH, NULL);
+        const char *override;
+
+        override = g_getenv ("GNOME22_USER_DIR");
+
+        if (override)
+               return g_build_filename (override,
+                                        PANEL_LAUNCHER_PERSONAL_PATH, NULL);
+        else
+                return g_build_filename (g_get_home_dir (), ".gnome2",
+                                         PANEL_LAUNCHER_PERSONAL_PATH, NULL);
 }
 
 gboolean
