@@ -18,6 +18,7 @@
 #include <glib/gi18n.h>
 #include <gio/gio.h>
 #include <gdk/gdkkeysyms.h>
+#include <gtk/gtkx.h>
 
 #include <libpanel-util/panel-glib.h>
 
@@ -392,9 +393,9 @@ panel_key_press_event (GtkWidget   *widget,
 	 * activating the key bindings here.
 	 */ 
 	if (GTK_IS_SOCKET (gtk_window_get_focus (GTK_WINDOW (widget))) &&
-	    event->keyval == GDK_F10 &&
+	    event->keyval == GDK_KEY_F10 &&
 	    (event->state & gtk_accelerator_get_default_mod_mask ()) == GDK_CONTROL_MASK)
-		return gtk_bindings_activate (GTK_OBJECT (widget),
+		return gtk_bindings_activate (G_OBJECT (widget),
 					      event->keyval,
 					      event->state);
 
@@ -426,7 +427,7 @@ static gboolean
 set_background_color (PanelToplevel *toplevel,
 		      guint16       *dropped)
 {
-	PanelColor color;
+	GdkRGBA color;
 
 	if (!dropped)
 		return FALSE;
@@ -435,10 +436,10 @@ set_background_color (PanelToplevel *toplevel,
 	     ! panel_profile_is_writable_background_color (toplevel))
 		return FALSE;
 
-	color.gdk.red   = dropped [0];
-	color.gdk.green = dropped [1];
-	color.gdk.blue  = dropped [2];
-	color.alpha     = 65535;
+	color.red   = dropped [0] / 65535.;
+	color.green = dropped [1] / 65535.;
+	color.blue  = dropped [2] / 65535.;
+	color.alpha = 1.;
 
 	panel_profile_set_background_color (toplevel, &color);
 	panel_profile_set_background_type (toplevel, PANEL_BACK_COLOR);
@@ -571,15 +572,42 @@ drop_nautilus_desktop_uri (PanelWidget *panel,
 	if (strncmp (basename, "trash", strlen ("trash")) == 0)
 		panel_applet_frame_create (panel->toplevel, pos,
 					   "OAFIID:GNOME_Panel_TrashApplet");
-	else if (strncmp (basename, "home", strlen ("home")) == 0)
-		panel_launcher_create_with_id (id, pos,
-					       "nautilus-home.desktop");
-	else if (strncmp (basename, "computer", strlen ("computer")) == 0)
-		panel_launcher_create_with_id (id, pos,
-					       "nautilus-computer.desktop");
+	else if (strncmp (basename, "home", strlen ("home")) == 0) {
+		char  *name;
+		char  *uri;
+		GFile *file;
+
+		file = g_file_new_for_path (g_get_home_dir ());
+		uri = g_file_get_uri (file);
+		name = panel_util_get_label_for_uri (uri);
+		g_free (uri);
+		g_object_unref (file);
+
+		panel_launcher_create_from_info (panel->toplevel,
+						 pos,
+						 TRUE, /* is_exec? */
+						 "nautilus --no-desktop", /* exec */
+						 name, /* name */
+						 _("Open your personal folder"), /* comment */
+						 PANEL_ICON_HOME); /* icon name */
+
+		g_free (name);
+	} else if (strncmp (basename, "computer", strlen ("computer")) == 0)
+		panel_launcher_create_from_info (panel->toplevel,
+						 pos,
+						 TRUE, /* is_exec? */
+						 "nautilus --no-desktop computer://", /* exec */
+						 _("Computer"), /* name */
+						 _("Browse all local and remote disks and folders accessible from this computer"), /* comment */
+						 PANEL_ICON_COMPUTER); /* icon name */
 	else if (strncmp (basename, "network", strlen ("network")) == 0)
-		panel_launcher_create_with_id (id, pos,
-					       "nautilus-scheme.desktop");
+		panel_launcher_create_from_info (panel->toplevel,
+						 pos,
+						 TRUE, /* is_exec? */
+						 "nautilus --no-desktop network://", /* exec */
+						 _("Network"), /* name */
+						 _("Browse bookmarked and local network locations"), /* comment */
+						 PANEL_ICON_NETWORK); /* icon name */
 	else
 		success = FALSE;
 
@@ -592,10 +620,18 @@ drop_urilist (PanelWidget *panel,
 	      char        *urilist)
 {
 	char     **uris;
+	GFile     *home;
+	GFile     *trash;
+	GFile     *computer;
+	GFile     *network;
 	gboolean   success;
 	int        i;
 
 	uris = g_uri_list_extract_uris (urilist);
+	home = g_file_new_for_path (g_get_home_dir ());
+	trash = g_file_new_for_uri ("trash://");
+	computer = g_file_new_for_uri ("computer://");
+	network = g_file_new_for_uri ("network://");
 
 	success = TRUE;
 	for (i = 0; uris[i]; i++) {
@@ -627,6 +663,25 @@ drop_urilist (PanelWidget *panel,
 		}
 
 		file = g_file_new_for_uri (uri);
+
+		if (g_file_equal (home, file)) {
+			success = drop_nautilus_desktop_uri (panel, pos, "x-nautilus-desktop:///home");
+			g_object_unref (file);
+			continue;
+		} else if (g_file_equal (trash, file)) {
+			success = drop_nautilus_desktop_uri (panel, pos, "x-nautilus-desktop:///trash");
+			g_object_unref (file);
+			continue;
+		} else if (g_file_equal (computer, file)) {
+			success = drop_nautilus_desktop_uri (panel, pos, "x-nautilus-desktop:///computer");
+			g_object_unref (file);
+			continue;
+		} else if (g_file_equal (network, file)) {
+			success = drop_nautilus_desktop_uri (panel, pos, "x-nautilus-desktop:///network");
+			g_object_unref (file);
+			continue;
+		}
+
 		info = g_file_query_info (file,
 					  "standard::type,"
 					  "standard::content-type,"
@@ -683,6 +738,10 @@ drop_urilist (PanelWidget *panel,
 		g_object_unref (file);
 	}
 
+	g_object_unref (home);
+	g_object_unref (trash);
+	g_object_unref (computer);
+	g_object_unref (network);
 	g_strfreev (uris);
 
 	return success;
